"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMonths = getMonths;
exports.filterCustomClasses = exports.calendarMonthsToRender = exports.getCurrentMonthIndex = exports.calendarDayNames = exports.calendarDaysToRender = exports.lastDayOfPrevMonth = exports.daysFromPrevMonth = void 0;

var _mapObjIndexed2 = _interopRequireDefault(require("ramda/src/mapObjIndexed"));

var _prop2 = _interopRequireDefault(require("ramda/src/prop"));

var _pickBy2 = _interopRequireDefault(require("ramda/src/pickBy"));

var _drop2 = _interopRequireDefault(require("ramda/src/drop"));

var _length2 = _interopRequireDefault(require("ramda/src/length"));

var _head2 = _interopRequireDefault(require("ramda/src/head"));

var _findIndex2 = _interopRequireDefault(require("ramda/src/findIndex"));

var _isEmpty2 = _interopRequireDefault(require("ramda/src/isEmpty"));

var _identity2 = _interopRequireDefault(require("ramda/src/identity"));

var _times2 = _interopRequireDefault(require("ramda/src/times"));

var _map2 = _interopRequireDefault(require("ramda/src/map"));

var _compose2 = _interopRequireDefault(require("ramda/src/compose"));

var _take2 = _interopRequireDefault(require("ramda/src/take"));

var _concat2 = _interopRequireDefault(require("ramda/src/concat"));

var _dropWhile2 = _interopRequireDefault(require("ramda/src/dropWhile"));

var _startOfMonth = _interopRequireDefault(require("date-fns/startOfMonth"));

var _endOfMonth = _interopRequireDefault(require("date-fns/endOfMonth"));

var _eachDayOfInterval = _interopRequireDefault(require("date-fns/eachDayOfInterval"));

var _lastDayOfMonth = _interopRequireDefault(require("date-fns/lastDayOfMonth"));

var _subDays = _interopRequireDefault(require("date-fns/subDays"));

var _getDay = _interopRequireDefault(require("date-fns/getDay"));

var _addMonths = _interopRequireDefault(require("date-fns/addMonths"));

var _subMonths = _interopRequireDefault(require("date-fns/subMonths"));

var _isSameMonth = _interopRequireDefault(require("date-fns/isSameMonth"));

var _differenceInCalendarMonths = _interopRequireDefault(require("date-fns/differenceInCalendarMonths"));

var _isBefore = _interopRequireDefault(require("date-fns/isBefore"));

var _isAfter = _interopRequireDefault(require("date-fns/isAfter"));

var _isSameDay = _interopRequireDefault(require("date-fns/isSameDay"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// TODO: separate util files
// CalendarMonth utils
// return days to prepend (these days are from previous month)
var daysFromPrevMonth = function daysFromPrevMonth(start, lastDayOfPrevMonth, firstDay) {
  // it's firtst day of the month, no need to prepend
  if ((0, _getDay.default)(start) === firstDay) {
    return [];
  } // otherwise prepend days from previous month


  var prevMonthDays = (0, _eachDayOfInterval.default)({
    start: (0, _subDays.default)(lastDayOfPrevMonth, 5),
    end: lastDayOfPrevMonth
  });
  return (0, _dropWhile2.default)(function (date) {
    return (0, _getDay.default)(date) !== firstDay;
  }, prevMonthDays);
};

exports.daysFromPrevMonth = daysFromPrevMonth;

var lastDayOfPrevMonth = function lastDayOfPrevMonth(month) {
  return (0, _lastDayOfMonth.default)((0, _subMonths.default)(month, 1));
}; // render prepended days from previous month + days from current month


exports.lastDayOfPrevMonth = lastDayOfPrevMonth;

var calendarDaysToRender = function calendarDaysToRender(firstDay, month) {
  // start of month
  var start = (0, _startOfMonth.default)(month); // end of month

  var end = (0, _endOfMonth.default)(month); // concant days from previous month and days from current month

  return (0, _concat2.default)(daysFromPrevMonth(start, lastDayOfPrevMonth(month), firstDay), (0, _eachDayOfInterval.default)({
    start: start,
    end: end
  }));
};

exports.calendarDaysToRender = calendarDaysToRender;

var calendarDayNames = function calendarDayNames(months) {
  return (0, _take2.default)(7, months);
}; // TODO: separate util files
// Calendar utils

/**
 * Will return months between `firstMonth` and `lastMonth`
 * @param {Date} firstMonth
 * @param {Date} lastMonth
 * @returns {Date[]}
 */


exports.calendarDayNames = calendarDayNames;

function getMonths(firstMonth, lastMonth) {
  return (0, _compose2.default)((0, _map2.default)(function (n) {
    return (0, _addMonths.default)((0, _startOfMonth.default)(firstMonth), n);
  }), (0, _times2.default)(_identity2.default) // add one month, because differenceInCalendarMonths will return n - 1
  )((0, _differenceInCalendarMonths.default)((0, _addMonths.default)(lastMonth, 1), firstMonth));
}

var getCurrentMonthIndex = function getCurrentMonthIndex(firstMonth, lastMonth, dates, future, visibleMonths) {
  var today = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : new Date();
  var datesOrToday = (0, _isEmpty2.default)(dates) ? [today] : dates;
  var months = getMonths(firstMonth, lastMonth);
  var index = (0, _findIndex2.default)( // $FlowExpected
  function (month) {
    return (0, _isSameMonth.default)(month, (0, _head2.default)(datesOrToday));
  }, months); // without future and without preselect

  if (!future && (0, _isEmpty2.default)(dates)) {
    return index - (visibleMonths - 1);
  } // with preselect


  if (!(0, _isEmpty2.default)(dates)) {
    var toSubstract = index + visibleMonths - (0, _length2.default)(months);

    if (toSubstract >= 0) {
      return index - toSubstract;
    }
  }

  return index;
};

exports.getCurrentMonthIndex = getCurrentMonthIndex;

var calendarMonthsToRender = function calendarMonthsToRender(visibleMonths, currentMonth, months) {
  if (visibleMonths >= months.length) {
    return months;
  }

  return (0, _compose2.default)((0, _take2.default)(visibleMonths), (0, _drop2.default)(currentMonth))(months);
};

exports.calendarMonthsToRender = calendarMonthsToRender;

var filterCustomClasses = function filterCustomClasses(from, to) {
  return (0, _compose2.default)((0, _pickBy2.default)((0, _prop2.default)("length")), (0, _mapObjIndexed2.default)(function (dates) {
    return dates.filter(function (date) {
      return to ? (0, _isSameDay.default)(date, from) || (0, _isSameDay.default)(date, to) || (0, _isBefore.default)(date, to) && (0, _isAfter.default)(date, from) : (0, _isSameDay.default)(date, from);
    });
  }));
};

exports.filterCustomClasses = filterCustomClasses;